
/*
*  TiledBackgroundPainter.java
*
*  Created on 31 July 2001, 16:22
*/
package jfreerails.client;
import java.awt.*;

/**
*  This abstract class stores a buffer of the backgound of the current visible
*  rectangle of the map. Code that is independent of how tiles are represented,
*  e.g. whether they are square or isometric, should go here.
*
*@author     Luke Lindsay
*@created    06 October 2001
*@version    1.0 N.B. Based on code from javagaming.org Scroller by Jeff
*      Kesselman.
*/


public abstract class TiledBackgroundPainter implements TiledBackgroundPainterInterface {

    
    /**
    *  This is used to create images that are compatible with the default
    *  graphics configuration. Such images can be drawn to the quickly screen
    *  since no conversion is needed.
    */
    protected java.awt.GraphicsConfiguration defaultConfiguration = java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();

    
    /**
    *  Used to draw on the backbuffer
    */
    protected java.awt.Graphics bg;

    
    /**
    *  The bounds and location of the map region that is stored in the
    *  offscreen Image backgraoundBuffer
    */
    protected Rectangle bufferRect = new Rectangle();

    
    /**
    *  An offscreen image storing the background of a region of the map
    */
    protected java.awt.Image backgroundBuffer;
    
    /**
    *  Description of the Method
    *
    *@param  g       Description of Parameter
    *@param  x       Description of Parameter
    *@param  y       Description of Parameter
    *@param  width   Description of Parameter
    *@param  height  Description of Parameter
    */
    
    public void paintBackground( java.awt.Graphics g, int x, int y, int width, int height ) {
        Rectangle  r = g.getClipBounds();
        if( ( bufferRect.width == r.width ) && ( bufferRect.height == r.height ) ) {
            g.drawImage( backgroundBuffer, x, y, x + width, y + height, x, y, x + width, y + height, null );
        }
        else {
            paintBufferRectangle( r.x, r.y, r.width, r.height );
            g.drawImage( backgroundBuffer, x, y, x + width, y + height, x, y, x + width, y + height, null );
        }
    }
    
    /**
    *  Updates the backbuffer as necessay, then draws it on to the Graphics
    *  object passed.
    *
    *@param  outputGraphics          Once it has been updated, the backbuffer
    *      is drawn onto this Graphics object.
    *@param  newVisibleRectectangle  The region of the map that the backbuffer
    *      must be updated to display.
    */
    
    public void paint( java.awt.Graphics outputGraphics, Rectangle newVisibleRectectangle ) {
        
        /*
        *  If this is the first call to the paint method or the component has just been resized,
        *  we need to create a new backgroundBuffer.
        */
        if( ( backgroundBuffer == null ) || ( newVisibleRectectangle.height != bufferRect.height ) || ( newVisibleRectectangle.width != bufferRect.width ) ) {
            setbackgroundBuffer( newVisibleRectectangle.width, newVisibleRectectangle.height );
        }
        
        /*
        *  Has the VisibleRectangle moved since the last paint?
        */
        if( ( bufferRect.x != newVisibleRectectangle.x ) || ( bufferRect.y != newVisibleRectectangle.y ) ) {
            int  dx = bufferRect.x - newVisibleRectectangle.x;
            int  dy = bufferRect.y - newVisibleRectectangle.y;
            scrollbackgroundBuffer( dx, dy );
            bufferRect.setBounds( newVisibleRectectangle );
        }
        if( ( bufferRect.width != newVisibleRectectangle.width ) && ( bufferRect.height != newVisibleRectectangle.height ) ) {
            paintBufferRectangle( newVisibleRectectangle.x - bufferRect.x, newVisibleRectectangle.y - bufferRect.y, newVisibleRectectangle.width, newVisibleRectectangle.height );
        }
        outputGraphics.drawImage( backgroundBuffer, newVisibleRectectangle.x, newVisibleRectectangle.y, null );
        bufferRect.setBounds( newVisibleRectectangle );
    }
    
    /**
    *  Description of the Method
    */
    
    public void refreshBackground() {
        paintBufferRectangle( 0, 0, bufferRect.width, bufferRect.height );
    }
    
    /**
    *  Changes the size of the backbuffer
    *
    *@param  w  buffer width
    *@param  h  buffer height
    */
    
    public void setbackgroundBuffer( int w, int h ) {
        backgroundBuffer = defaultConfiguration.createCompatibleImage( w, h );
        bufferRect.height = backgroundBuffer.getHeight( null );
        bufferRect.width = backgroundBuffer.getWidth( null );
        bg = backgroundBuffer.getGraphics();
        bg.clearRect( 0, 0, w, h );
        refreshBackground();
    }
    
    /**
    *  Description of the Method
    *
    *@param  x       Description of Parameter
    *@param  y       Description of Parameter
    *@param  width   Description of Parameter
    *@param  height  Description of Parameter
    */
    
    public abstract void paintBufferRectangle( int x, int y, int width, int height );
    
    /**
    *  Used for benchmarking only.
    *
    *@param  g                    The bb is drawn onto this Graphics object.
    *@param  newVisibleRectangle  The bb is updated to display this region of
    *      the map.
    */
    
    public void benchmark( Graphics g, Rectangle newVisibleRectangle ) {
        
        /*
        *  If this is the first call to the paint method or the component has just been resized,
        *  we need to create a new backgroundBuffer.
        */
        if( ( backgroundBuffer == null ) || ( newVisibleRectangle.height != bufferRect.height ) || ( newVisibleRectangle.width != bufferRect.width ) ) {
            setbackgroundBuffer( newVisibleRectangle.width, newVisibleRectangle.height );
        }
        
        /*
        *  Has the VisibleRectangle moved since the last paint?
        */
        if( ( bufferRect.x != newVisibleRectangle.x ) || ( bufferRect.y != newVisibleRectangle.y ) ) {
            int  dx = bufferRect.x - newVisibleRectangle.x;
            int  dy = bufferRect.y - newVisibleRectangle.y;
            scrollbackgroundBuffer( dx, dy );
            bufferRect.setBounds( newVisibleRectangle );
        }
        if( ( bufferRect.width != newVisibleRectangle.width ) && ( bufferRect.height != newVisibleRectangle.height ) ) {
            paintBufferRectangle( newVisibleRectangle.x - bufferRect.x, newVisibleRectangle.y - bufferRect.y, newVisibleRectangle.width, newVisibleRectangle.height );
        }
        
        //g.setClip(0,0,400,400);
        
        //g.drawImage( backgroundBuffer, 0, 0, null );
        g.setClip( 0, 0, 400, 400 );
        g.drawImage( backgroundBuffer, 0, 0, null );
        g.setClip( 0, 400, 400, 400 );
        g.drawImage( backgroundBuffer, 0, 0, null );
        g.setClip( 400, 0, 400, 400 );
        g.drawImage( backgroundBuffer, 0, 0, null );
        g.setClip( 400, 400, 400, 400 );
        g.drawImage( backgroundBuffer, 0, 0, null );
        bufferRect.setBounds( newVisibleRectangle );
    }
    
    /**
    *  Description of the Method
    *
    *@param  mapCoord  Description of Parameter
    */
    
    public abstract void updateTile( Point mapCoord );
    
    /**
    *  Description of the Method
    *
    *@param  dx  Description of Parameter
    *@param  dy  Description of Parameter
    */
    
    protected void scrollbackgroundBuffer( int dx, int dy ) {
        int  copyWidth = bufferRect.width;
        int  copyHeight = bufferRect.height;
        int  copySourceX = 0;
        int  copySourceY = 0;
        if( dx > 0 ) {
            copyWidth -= dx;
        }
        else {
            copyWidth += dx;
            copySourceX = -dx;
        }
        if( dy > 0 ) {
            copyHeight -= dy;
        }
        else {
            copyHeight += dy;
            copySourceY = -dy;
        }
        bg.copyArea( copySourceX, copySourceY, copyWidth, copyHeight, dx, dy );
        bufferRect.x -= dx;
        bufferRect.y -= dy;
        
        // paint exposed areas
        if( dx > 0 ) {
            bg.setClip( 0, 0, dx, bufferRect.height );
            bg.clearRect( 0, 0, dx, bufferRect.height );
            paintBufferRectangle( 0, 0, dx, bufferRect.height );
        }
        else {
            if( dx < 0 ) {
                bg.setClip( bufferRect.width + dx, 0, -dx, bufferRect.height );
                bg.clearRect( bufferRect.width + dx, 0, -dx, bufferRect.height );
                paintBufferRectangle( bufferRect.width + dx, 0, -dx, bufferRect.height );
            }
        }
        if( dy > 0 ) {
            bg.setClip( 0, 0, bufferRect.width, dy );
            bg.clearRect( 0, 0, bufferRect.width, dy );
            paintBufferRectangle( 0, 0, bufferRect.width, dy );
        }
        else {
            if( dy < 0 ) {
                bg.setClip( 0, bufferRect.height + dy, bufferRect.width, -dy );
                bg.clearRect( 0, bufferRect.height + dy, bufferRect.width, -dy );
                paintBufferRectangle( 0, bufferRect.height + dy, bufferRect.width, -dy );
            }
        }
        bg.setClip( 0, 0, bufferRect.width, bufferRect.height );
    }
}
